------------------------------------------------------------------------
-- Event:        Community Days 2014, Roma 23-25 Settembre 2014
-- Session:      Keynote
-- Demo:         Setup Hekaton
-- Author:       Sergio Govoni
-- Notes:        --
------------------------------------------------------------------------

USE [master];
GO

--------------------------------------------------------------------------
-- Hekaton In-Memory OLTP
--------------------------------------------------------------------------
/*
IF EXISTS(SELECT * FROM sys.databases WHERE NAME = 'CommunityDays')
BEGIN
  ALTER DATABASE [CommunityDays]
    SET SINGLE_USER WITH ROLLBACK IMMEDIATE;

  DROP DATABASE [CommunityDays];
END;
GO

-- Creazione database CommunityDays
CREATE DATABASE CommunityDays
  ON PRIMARY
  (
    NAME = [CommunityDays_Data]
	,FILENAME = 'C:\SQL\DBs\CommunityDays_Data.mdf'
	,SIZE = 200MB
  )
  LOG ON
  (
    NAME = [CommunityDays_Log]
	,FILENAME = 'C:\SQL\DBs\CommunityDays_Log.ldf'
	,SIZE = 200MB
  );
GO


-- Abilito il database CommunityDays a contenere In-Memory optimized table
ALTER DATABASE CommunityDays
  ADD FILEGROUP CommunityDays_In_Memory CONTAINS MEMORY_OPTIMIZED_DATA;
GO


ALTER DATABASE CommunityDays
  ADD FILE
  (
    NAME = [CommunityDays_In_Memory]
	,FILENAME = 'C:\SQL\DBs\CommunityDays_In_Memory'
  )
  TO FILEGROUP CommunityDays_In_Memory;
GO
*/


IF EXISTS(SELECT * FROM sys.databases WHERE NAME = 'CommunityDaysERP')
BEGIN
  ALTER DATABASE [CommunityDaysERP]
    SET SINGLE_USER WITH ROLLBACK IMMEDIATE;

  DROP DATABASE [CommunityDaysERP];
END;
GO

-- Creazione database CommunityDaysERP
CREATE DATABASE CommunityDaysERP
  ON PRIMARY
  (
    NAME = [CommunityDaysERP_Data]
	,FILENAME = 'C:\SQL\DBs\CommunityDaysERP_Data.mdf'
	,SIZE = 5MB
  )
  LOG ON
  (
    NAME = [CommunityDaysERP_Log]
	,FILENAME = 'C:\SQL\DBs\CommunityDaysERP_Log.ldf'
	,SIZE = 1MB
  );
GO


USE [CommunityDays];
GO


--------------------------------------------------------------------------
-- Create Tables
--------------------------------------------------------------------------

IF OBJECT_ID('dbo.usp_Insert_SalesOrderHeaderH_C_SD', 'P') IS NOT NULL
  DROP PROCEDURE dbo.usp_Insert_SalesOrderHeaderH_C_SD;
GO

IF OBJECT_ID('dbo.usp_Insert_SalesOrderHeaderH_C_SO', 'P') IS NOT NULL
  DROP PROCEDURE dbo.usp_Insert_SalesOrderHeaderH_C_SO;
GO

IF OBJECT_ID('dbo.usp_SaveElapsedTime', 'P') IS NOT NULL
  DROP PROCEDURE dbo.usp_SaveElapsedTime;
GO

IF OBJECT_ID('dbo.ElapsedTime') IS NOT NULL
  DROP TABLE dbo.ElapsedTime;
GO
CREATE TABLE dbo.ElapsedTime
(
  ID INTEGER IDENTITY(1, 1) NOT NULL
  ,SPName SYSNAME NOT NULL
  ,NRows INTEGER DEFAULT 100000
  ,StartTime DATETIME NOT NULL
  ,EndTime DATETIME NOT NULL
  CONSTRAINT PK_ElapsedTime PRIMARY KEY NONCLUSTERED HASH (ID) WITH (BUCKET_COUNT = 100)
)
WITH (MEMORY_OPTIMIZED = ON, DURABILITY = SCHEMA_AND_DATA);
GO


IF OBJECT_ID('dbo.SalesOrderHeaderDisk') IS NOT NULL
  DROP TABLE dbo.SalesOrderHeaderDisk;
GO
CREATE TABLE dbo.SalesOrderHeaderDisk
(
  OrderId INTEGER IDENTITY(1, 1) NOT NULL
  ,OrderDate DATETIME NOT NULL
  ,OrderNumber UNIQUEIDENTIFIER NOT NULL
  ,OrderStatus SMALLINT NOT NULL
  CONSTRAINT PK_SalesOrderHeader PRIMARY KEY (OrderId)
);
GO

IF OBJECT_ID('dbo.SalesOrderHeaderHSD') IS NOT NULL
  DROP TABLE dbo.SalesOrderHeaderHSD;
GO
CREATE TABLE dbo.SalesOrderHeaderHSD
(
  OrderId INTEGER IDENTITY(1, 1) NOT NULL
  ,OrderDate DATETIME NOT NULL
  ,OrderNumber UNIQUEIDENTIFIER NOT NULL
  ,OrderStatus SMALLINT NOT NULL
  CONSTRAINT PK_SalesOrderHeaderHSD PRIMARY KEY NONCLUSTERED HASH (OrderId) WITH (BUCKET_COUNT = 100000)
)
WITH (MEMORY_OPTIMIZED = ON, DURABILITY = SCHEMA_AND_DATA);
GO


IF OBJECT_ID('dbo.SalesOrderHeaderHSO') IS NOT NULL
  DROP TABLE dbo.SalesOrderHeaderHSO;
GO
CREATE TABLE dbo.SalesOrderHeaderHSO
(
  OrderId INTEGER IDENTITY(1, 1) NOT NULL
  ,OrderDate DATETIME NOT NULL
  ,OrderNumber UNIQUEIDENTIFIER NOT NULL
  ,OrderStatus SMALLINT NOT NULL
  CONSTRAINT PK_SalesOrderHeaderHSO PRIMARY KEY NONCLUSTERED HASH (OrderId) WITH (BUCKET_COUNT = 100000)
)
WITH (MEMORY_OPTIMIZED = ON, DURABILITY = SCHEMA_ONLY);
GO


IF OBJECT_ID('dbo.SalesOrderHeaderH_C_SD') IS NOT NULL
  DROP TABLE dbo.SalesOrderHeaderH_C_SD;
GO
CREATE TABLE dbo.SalesOrderHeaderH_C_SD
(
  OrderId INTEGER IDENTITY(1, 1) NOT NULL
  ,OrderDate DATETIME NOT NULL
  ,OrderNumber UNIQUEIDENTIFIER NOT NULL
  ,OrderStatus SMALLINT NOT NULL
  CONSTRAINT PK_SalesOrderHeaderH_C_SD PRIMARY KEY NONCLUSTERED HASH (OrderId) WITH (BUCKET_COUNT = 100000)
)
WITH (MEMORY_OPTIMIZED = ON, DURABILITY = SCHEMA_AND_DATA);
GO


IF OBJECT_ID('dbo.SalesOrderHeaderH_C_SO') IS NOT NULL
  DROP TABLE dbo.SalesOrderHeaderH_C_SO;
GO
CREATE TABLE dbo.SalesOrderHeaderH_C_SO
(
  OrderId INTEGER IDENTITY(1, 1) NOT NULL
  ,OrderDate DATETIME NOT NULL
  ,OrderNumber UNIQUEIDENTIFIER NOT NULL
  ,OrderStatus SMALLINT NOT NULL
  CONSTRAINT PK_SalesOrderHeaderH_C_SO PRIMARY KEY NONCLUSTERED HASH (OrderId) WITH (BUCKET_COUNT = 100000)
)
WITH (MEMORY_OPTIMIZED = ON, DURABILITY = SCHEMA_ONLY);
GO


--------------------------------------------------------------------------
-- Creates Stored Procedures
--------------------------------------------------------------------------

IF OBJECT_ID('dbo.usp_SaveElapsedTime', 'P') IS NOT NULL
  DROP PROCEDURE dbo.usp_SaveElapsedTime;
GO
CREATE PROCEDURE dbo.usp_SaveElapsedTime
(
  @SPName SYSNAME
  ,@StartTime DATETIME
  ,@EndTime DATETIME
) WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN
  ATOMIC WITH
  (
    TRANSACTION ISOLATION LEVEL = SNAPSHOT
    ,LANGUAGE = 'us_english'
  );
  
  INSERT INTO dbo.ElapsedTime(SPName, StartTime, EndTime)
  VALUES (@SPName, @StartTime, @EndTime);
END;
GO


-- SP for insert on Disk-based table
IF OBJECT_ID('dbo.usp_Insert_SalesOrderHeaderDisk', 'P') IS NOT NULL
  DROP PROCEDURE dbo.usp_Insert_SalesOrderHeaderDisk;
GO
CREATE PROCEDURE dbo.usp_Insert_SalesOrderHeaderDisk AS
BEGIN
  SET NOCOUNT ON;

  DECLARE
    @ID INTEGER = 1
	,@Status SMALLINT = 1
	,@Start DATETIME = GETDATE()
	,@End DATETIME;

  WHILE (@ID <= 10000)
  BEGIN
    INSERT INTO dbo.SalesOrderHeaderDisk
	(
	  --OrderId
	  OrderDate
	  ,OrderNumber
	  ,OrderStatus
	)
	VALUES
	(
	  --@ID
	  GETDATE()
	  ,NEWID()
	  ,@Status
	);
    SET @ID = @ID + 1;
  END;

  SET @End = GETDATE();

  EXEC dbo.usp_SaveElapsedTime @SPName='usp_Insert_SalesOrderHeaderDisk', @StartTime=@Start, @EndTime=@End;

  SET NOCOUNT OFF;
END;
GO


-- SP for insert on In-Memory table, schema and data
IF OBJECT_ID('dbo.usp_Insert_SalesOrderHeaderHSD', 'P') IS NOT NULL
  DROP PROCEDURE dbo.usp_Insert_SalesOrderHeaderHSD;
GO
CREATE PROCEDURE dbo.usp_Insert_SalesOrderHeaderHSD AS
BEGIN
  SET NOCOUNT ON;

  DECLARE
    @ID INTEGER = 1
	,@Status SMALLINT = 1
	,@Start DATETIME = GETDATE()
	,@End DATETIME;

  WHILE (@ID <= 10000)
  BEGIN
    INSERT INTO dbo.SalesOrderHeaderHSD
	(
	  --OrderId
	  OrderDate
	  ,OrderNumber
	  ,OrderStatus
	)
	VALUES
	(
	  --@ID
	  GETDATE()
	  ,NEWID()
	  ,@Status
	);
    SET @ID = @ID + 1;
  END;

  SET @End = GETDATE();

  EXEC dbo.usp_SaveElapsedTime @SPName='usp_Insert_SalesOrderHeaderHSD', @StartTime=@Start, @EndTime=@End;

  SET NOCOUNT OFF;
END;
GO


-- SP for insert on In-Memory table, schema only
IF OBJECT_ID('dbo.usp_Insert_SalesOrderHeaderHSO', 'P') IS NOT NULL
  DROP PROCEDURE dbo.usp_Insert_SalesOrderHeaderHSO;
GO
CREATE PROCEDURE dbo.usp_Insert_SalesOrderHeaderHSO AS
BEGIN
  SET NOCOUNT ON;

  DECLARE
    @ID INTEGER = 1
	,@Status SMALLINT = 1
	,@Start DATETIME = GETDATE()
	,@End DATETIME;

  WHILE (@ID <= 10000)
  BEGIN
    INSERT INTO dbo.SalesOrderHeaderHSO
	(
	  --OrderId
	  OrderDate
	  ,OrderNumber
	  ,OrderStatus
	)
	VALUES
	(
	  --@ID
	  GETDATE()
	  ,NEWID()
	  ,@Status
	);
    SET @ID = @ID + 1;
  END;

  SET @End = GETDATE();

  EXEC dbo.usp_SaveElapsedTime @SPName='usp_Insert_SalesOrderHeaderHSO', @StartTime=@Start, @EndTime=@End;

  SET NOCOUNT OFF;
END;
GO


-- SP for insert on In-Memory table, schema and data, compiled stored procedure
CREATE PROCEDURE dbo.usp_Insert_SalesOrderHeaderH_C_SD
  WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
  AS
BEGIN
  ATOMIC WITH
  (
    TRANSACTION ISOLATION LEVEL = SNAPSHOT
    ,LANGUAGE = 'us_english'
  );

  DECLARE
    @ID INTEGER = 1
	,@Status SMALLINT = 1
	,@Start DATETIME = GETDATE()
	,@End DATETIME;

  WHILE (@ID <= 10000)
  BEGIN
    INSERT INTO dbo.SalesOrderHeaderH_C_SD
	(
	  --OrderId
	  OrderDate
	  ,OrderNumber
	  ,OrderStatus
	)
	VALUES
	(
	  --@ID
	  GETDATE()
	  ,NEWID()
	  ,@Status
	);
    SET @ID = @ID + 1;
  END;

  SET @End = GETDATE();

  INSERT INTO dbo.ElapsedTime(SPName, StartTime, EndTime)
  VALUES ('usp_Insert_SalesOrderHeaderH_C_SD', @Start, @End);
END;
GO


-- SP for insert on In-Memory table, schema only, compiled stored procedure
CREATE PROCEDURE dbo.usp_Insert_SalesOrderHeaderH_C_SO
  WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
  AS
BEGIN
  ATOMIC WITH
  (
    TRANSACTION ISOLATION LEVEL = SNAPSHOT
    ,LANGUAGE = 'us_english'
  );

  DECLARE
    @ID INTEGER = 1
	,@Status SMALLINT = 1
	,@Start DATETIME = GETDATE()
	,@End DATETIME;

  WHILE (@ID <= 10000)
  BEGIN
    INSERT INTO dbo.SalesOrderHeaderH_C_SO
	(
	  --OrderId
	  OrderDate
	  ,OrderNumber
	  ,OrderStatus
	)
	VALUES
	(
	  --@ID
	  GETDATE()
	  ,NEWID()
	  ,@Status
	);
    SET @ID = @ID + 1;
  END;

  SET @End = GETDATE();

  INSERT INTO dbo.ElapsedTime(SPName, StartTime, EndTime)
  VALUES ('usp_Insert_SalesOrderHeaderH_C_SO', @Start, @End);
END;
GO
