------------------------------------------------------------------------
-- Event:        Community Days 2014, Roma 23-25 Settembre 2014
-- Session:      Keynote
-- Demo:         Setup Columnstore indexes
-- Author:       Sergio Govoni
-- Notes:        --
------------------------------------------------------------------------

USE [AdventureWorksDW2014];
GO

--------------------------------------------------------------------------
-- Columnstore Indexes
--------------------------------------------------------------------------

/*
SELECT * FROM dbo.FactProductInventory;
*/

-- Creazione tabella dbo.FactProductInventoryXXL
IF OBJECT_ID('dbo.FactProductInventoryXXL') IS NOT NULL
  DROP TABLE dbo.FactProductInventoryXXL;
GO

CREATE TABLE dbo.FactProductInventoryXXL
(
  ID int identity(1, 1) NOT NULL
  ,ProductKey int NOT NULL
  ,DateKey int NOT NULL
  ,MovementDate date NOT NULL
  ,UnitCost money NOT NULL
  ,UnitsIn int NOT NULL
  ,UnitsOut int NOT NULL
  ,UnitsBalance int NOT NULL
) WITH (DATA_COMPRESSION = PAGE)
GO


-- Inserimento dati nella tabella dbo.FactProductInventoryXXL
INSERT INTO
  dbo.FactProductInventoryXXL
SELECT
  fpi.*
FROM
  dbo.FactProductInventory AS fpi
GO 20


-- Creazione indice cluster su dbo.FactProductInventoryXXL
ALTER TABLE dbo.FactProductInventoryXXL
  ADD CONSTRAINT PK_FactProductInventoryXXL
    PRIMARY KEY CLUSTERED (ID, ProductKey, DateKey);
GO


-- Creazione tabella dbo.FactProductInventoryXXL_CCI
IF OBJECT_ID('dbo.FactProductInventoryXXL_CCI') IS NOT NULL
  DROP TABLE dbo.FactProductInventoryXXL_CCI;
GO

CREATE TABLE dbo.FactProductInventoryXXL_CCI
(
  ID int identity(1, 1) NOT NULL
  ,ProductKey int NOT NULL
  ,DateKey int NOT NULL
  ,MovementDate date NOT NULL
  ,UnitCost money NOT NULL
  ,UnitsIn int NOT NULL
  ,UnitsOut int NOT NULL
  ,UnitsBalance int NOT NULL
) WITH (DATA_COMPRESSION = PAGE)
GO

-- Inserimento dati nella tabella dbo.FactProductInventoryXXL_CCI
INSERT INTO
  dbo.FactProductInventoryXXL_CCI
SELECT
  fpi.*
FROM
  dbo.FactProductInventory AS fpi
GO 20


-- Creazione cluster columnstore index
CREATE CLUSTERED COLUMNSTORE INDEX CCI_FactProductInventoryXXL_CCI
  ON dbo.FactProductInventoryXXL_CCI WITH (DATA_COMPRESSION = COLUMNSTORE_ARCHIVE);
GO


IF OBJECT_ID('dbo.vw_row_groups_state') IS NOT NULL
  DROP VIEW dbo.vw_row_groups_state;
GO

CREATE VIEW dbo.vw_row_groups_state
AS
  SELECT
    csrg.row_group_id
    ,csrg.delta_store_hobt_id
    ,csrg.state_description
    ,csrg.total_rows
    ,csrg.deleted_rows
    ,size_in_MB = (csrg.size_in_bytes/1000000.00)
	,csrg.object_id
    ,csrg.index_id
FROM
  sys.column_store_row_groups AS csrg;
GO