------------------------------------------------------------------------
-- Event:        Community Days 2014, Roma 23-25 Settembre 2014
-- Session:      Keynote
-- Demo:         Column Store Indexes in SQL Server 2014
-- Author:       Sergio Govoni
-- Notes:        --
------------------------------------------------------------------------

USE [AdventureWorksDW2014];


SELECT COUNT_BIG(*) FROM dbo.FactProductInventoryXXL;
SELECT COUNT_BIG(*) FROM dbo.FactProductInventoryXXL_CCI;
GO


EXEC sp_helpindex 'dbo.FactProductInventoryXXL';
EXEC sp_helpindex 'dbo.FactProductInventoryXXL_CCI';
GO


EXEC sp_spaceused 'dbo.FactProductInventoryXXL';
EXEC sp_spaceused 'dbo.FactProductInventoryXXL_CCI';
GO


-- Movimenti totalizzati per prodotto

SELECT
  ProductKey
  ,SUM(UnitsIn) SUnitsIn
  ,SUM(UnitsOut) SUnitsOut
FROM
  dbo.FactProductInventoryXXL
GROUP BY
  ProductKey
ORDER BY
  SUnitsIn DESC
GO


SELECT
  ProductKey
  ,SUM(UnitsIn) SUnitsIn
  ,SUM(UnitsOut) SUnitsOut
FROM
  dbo.FactProductInventoryXXL_CCI
GROUP BY
  ProductKey
ORDER BY
  SUnitsIn DESC
GO


-- Creazione cluster columnstore index
-- Non sono state specificate le colonne della tabella!
-- E' stata specificata la compressione aggiuntiva COLUMNSTORE_ARCHIVE
--CREATE CLUSTERED COLUMNSTORE INDEX CCI_FactProductInventoryXXL_CCI
--  ON dbo.FactProductInventoryXXL_CCI WITH (DATA_COMPRESSION = COLUMNSTORE_ARCHIVE);
--GO


SELECT * FROM sys.column_store_row_groups;
GO


--SELECT
--  csrg.row_group_id
--  ,csrg.delta_store_hobt_id
--  ,csrg.state_description
--  ,csrg.total_rows
--  ,csrg.deleted_rows
--  ,size_in_MB = (csrg.size_in_bytes/1000000.00)
--  ,csrg.object_id
--  ,csrg.index_id
--FROM
--  sys.column_store_row_groups AS csrg
--WHERE
--  OBJECT_NAME(csrg.object_id) = 'FactProductInventoryXXL_CCI';
--GO


-- In quanti row group  stata suddivisa la tabella ?
SELECT * FROM dbo.vw_row_groups_state
WHERE OBJECT_NAME([object_id]) = 'FactProductInventoryXXL_CCI';
GO


-- Inseriamo 3.000
SET NOCOUNT ON;
DECLARE @i INTEGER = 0;
WHILE (@i < 3000)
BEGIN
  INSERT INTO
    dbo.FactProductInventoryXXL_CCI
  SELECT
    fpi.*
  FROM
    dbo.FactProductInventory AS fpi
  ORDER BY
    ProductKey
	,DateKey
  OFFSET @i ROWS FETCH NEXT 1 ROW ONLY;
  SET @i = @i + 1;
END;
SET NOCOUNT OFF;


/*
SELECT TOP 500 * FROM dbo.FactProductInventoryXXL_CCI ORDER BY ID DESC;
GO
*/

-- Situazione dei rowgroup
SELECT * FROM dbo.vw_row_groups_state
WHERE OBJECT_NAME([object_id]) = 'FactProductInventoryXXL_CCI';
GO

-- Lo stato del nuovo row group  OPEN, la colonna total_rows contiene le righe
-- totali nel row group


-- Rebuild dell'indice
DBCC DBREINDEX('dbo.FactProductInventoryXXL_CCI', CCI_FactProductInventoryXXL_CCI);
GO


-- Situazione dei rowgroup
SELECT * FROM dbo.vw_row_groups_state
WHERE OBJECT_NAME([object_id]) = 'FactProductInventoryXXL_CCI';
GO

-- Le righe nel Delta Store, che era in stato OPEN, sono state ridistribuite
-- nei row group esistenti (con total_rows < 1048576)



-- Eliminazione righe
DELETE FROM dbo.FactProductInventoryXXL_CCI
WHERE (MovementDate = '20130301');
GO


-- Situazione dei rowgroup
SELECT * FROM dbo.vw_row_groups_state
WHERE OBJECT_NAME([object_id]) = 'FactProductInventoryXXL_CCI';
GO


-- La colonna deleted_rows indica, per row group, le righe eliminate
-- (gestite dalla Delete Bitmap)
SELECT * FROM dbo.FactProductInventoryXXL_CCI WHERE (MovementDate = '20130301');
GO


-- Rebuild dell'indice
DBCC DBREINDEX('dbo.FactProductInventoryXXL_CCI', CCI_FactProductInventoryXXL_CCI);
GO


-- Aggiornamento righe
UPDATE p SET p.UnitCost = (p.UnitCost + 0.30)
FROM dbo.FactProductInventoryXXL_CCI AS p
WHERE (p.MovementDate = '20130302');
GO


-- Situazione dei rowgroup
SELECT * FROM dbo.vw_row_groups_state
WHERE OBJECT_NAME([object_id]) = 'FactProductInventoryXXL_CCI';
GO


-- Rebuild dell'indice
DBCC DBREINDEX('dbo.FactProductInventoryXXL_CCI', CCI_FactProductInventoryXXL_CCI);
GO


-- Situazione dei rowgroup
SELECT * FROM dbo.vw_row_groups_state
WHERE OBJECT_NAME([object_id]) = 'FactProductInventoryXXL_CCI';
GO